import { DeepClient, SerialOperation } from '@deep-foundation/deeplinks/imports/client';
/**
  * Gets serial operations to insert {@link Package.DeviceRegistrationToken}
  *
  * @example
  * #### Insert {@link Package.DeviceRegistrationToken}
 ```ts
 const {serialOperations, linkIds} = await getDeviceRegistrationTokenInsertSerialOperations({
   deep
 });
 await deep.serial({
   operations: serialOperations
 })
 ```
   * #### Insert {@link Package.DeviceRegistrationToken} with reserved link id
  ```ts
  const reservedLinkIds = await deep.reserve(2);
  const deviceRegistrationTokenLinkId = reservedLinkIds.pop();
  const containLinkId = reservedLinkIds.pop();
  
  const {serialOperations, linkIds} = await getDeviceRegistrationTokenInsertSerialOperations({
    deep,
    deviceRegistrationToken: {
      title,
      body,
    },
    reservedLinkIds: {
      deviceRegistrationTokenLinkId,
      containLinkId,
    }
  });
  await deep.serial({
    operations: serialOperations
  })
  ```
  */
export declare function getDeviceRegistrationTokenInsertSerialOperations(param: GetDeviceRegistrationTokenInsertSerialOperationsParam): Promise<GetDeviceRegistrationTokenInsertSerialOperationsResult>;
export interface GetDeviceRegistrationTokenInsertSerialOperationsParam {
    /**
     * Reserved link ids that will be used in the serial operations
     */
    reservedLinkIds?: {
        /**
         * Reserved link id for the deviceRegistrationToken
         */
        deviceRegistrationTokenLinkId?: number;
        /**
         * Reserved link id for the contain
         */
        containLinkId?: number;
    };
    /**
     * Link ids of types that will be used in the serial operations
     */
    typeLinkIds?: {
        /**
         * Link id of the contain type
         */
        containTypeLinkId?: number;
        /**
         * Link id of the deviceRegistrationToken type
         */
        deviceRegistrationTokenTypeLinkId?: number;
    };
    /**
     * Deep Client
     */
    deep: DeepClient;
    /**
     * Device Registration Token
     */
    deviceRegistrationToken: string;
    /**
     * Link id of the container
     *
     * @remarks
     * If it is null, contain link will not be created
     * @defaultValue {@link GetDeviceRegistrationTokenInsertSerialOperationsParam.deep.linkId} if not provided or undefined
     */
    containerLinkId?: number | undefined | null;
    /**
     * Value of the contain link
     *
     * @remarks
     * If {@link GetDeviceRegistrationTokenInsertSerialOperationsParam.containerLinkId} is null, this will be ignored
     */
    containValue?: string | undefined;
}
export interface GetDeviceRegistrationTokenInsertSerialOperationsResult {
    serialOperations: Array<SerialOperation>;
    linkIds: Required<Exclude<GetDeviceRegistrationTokenInsertSerialOperationsParam['reservedLinkIds'], undefined>>;
}
