import { DeepClient, SerialOperation } from '@deep-foundation/deeplinks/imports/client';
/**
  * Gets serial operations to insert {@link Package.WebPushCertificate}
  *
  * @example
  * #### Insert {@link Package.WebPushCertificate}
 ```ts
 const {serialOperations, linkIds} = await getWebPushCertificateInsertSerialOperations({
   deep
 });
 await deep.serial({
   operations: serialOperations
 })
 ```
   * #### Insert {@link Package.WebPushCertificate} with reserved link id
  ```ts
  const reservedLinkIds = await deep.reserve(2);
  const webPushCertificateLinkId = reservedLinkIds.pop();
  const containLinkId = reservedLinkIds.pop();
  
  const {serialOperations, linkIds} = await getWebPushCertificateInsertSerialOperations({
    deep,
    webPushCertificate: {
      title,
      body,
    },
    reservedLinkIds: {
      webPushCertificateLinkId,
      containLinkId,
    }
  });
  await deep.serial({
    operations: serialOperations
  })
  ```
  */
export declare function getWebPushCertificateInsertSerialOperations(param: GetWebPushCertificateInsertSerialOperationsParam): Promise<GetWebPushCertificateInsertSerialOperationsResult>;
export interface GetWebPushCertificateInsertSerialOperationsParam {
    /**
     * Reserved link ids that will be used in the serial operations
     */
    reservedLinkIds?: {
        /**
         * Reserved link id for the webPushCertificate
         */
        webPushCertificateLinkId?: number;
        /**
         * Reserved link id for the contain
         */
        containForWebPushCertificateLinkId?: number;
        /**
         * Reserved link id for the usesWebPushCertificate
         */
        usesWebPushCertificateLinkId?: number;
        /**
         * Reserved link id for the contain for usesServiceAccount
         */
        containForUsesWebPushCertificateLinkId?: number;
    };
    /**
     * Link ids of types that will be used in the serial operations
     */
    typeLinkIds?: {
        /**
         * Link id of the contain type
         */
        containTypeLinkId?: number;
        /**
         * Link id of the webPushCertificate type
         */
        webPushCertificateTypeLinkId?: number;
        /**
         * Link id of the usesWebPushCertificate type
         */
        usesWebPushCertificateTypeLinkId?: number;
    };
    /**
     * Deep Client
     */
    deep: DeepClient;
    /**
     * Web Push Certificate
     */
    webPushCertificate: string;
    /**
     * Link id of the container
     *
     * @remarks
     * If it is null, contain link will not be created
     * @defaultValue {@link GetWebPushCertificateInsertSerialOperationsParam.deep.linkId} if not provided or undefined
     */
    containerLinkId?: number | undefined | null;
    /**
     * Value of the contain link
     *
     * @remarks
     * If {@link GetWebPushCertificateInsertSerialOperationsParam.containerLinkId} is null, this will be ignored
     */
    valueForContainForWebPushCertificate?: string | undefined;
    /**
     * Value of the contain link
     *
     * @remarks
     * If {@link GetWebPushCertificateInsertSerialOperationsParam.containerLinkId} is null, this will be ignored
     */
    valueForContainForUsesWebPushCertificate?: string | undefined;
    /**
     * If true, the link will be made active by creating a {@link Package.UsesServiceAccount} link pointing to it
     *
     * @defaultValue false
     */
    shouldMakeActive?: boolean;
}
export interface GetWebPushCertificateInsertSerialOperationsResult {
    serialOperations: Array<SerialOperation>;
    linkIds: Required<Exclude<GetWebPushCertificateInsertSerialOperationsParam['reservedLinkIds'], undefined>>;
}
