import { createSerialOperation } from '@deep-foundation/deeplinks/imports/gql';
import { Package } from './package';
/**
  * Gets serial operations to insert {@link Package.WebPushCertificate}
  *
  * @example
  * #### Insert {@link Package.WebPushCertificate}
 ```ts
 const {serialOperations, linkIds} = await getWebPushCertificateInsertSerialOperations({
   deep
 });
 await deep.serial({
   operations: serialOperations
 })
 ```
   * #### Insert {@link Package.WebPushCertificate} with reserved link id
  ```ts
  const reservedLinkIds = await deep.reserve(2);
  const webPushCertificateLinkId = reservedLinkIds.pop();
  const containLinkId = reservedLinkIds.pop();
  
  const {serialOperations, linkIds} = await getWebPushCertificateInsertSerialOperations({
    deep,
    webPushCertificate: {
      title,
      body,
    },
    reservedLinkIds: {
      webPushCertificateLinkId,
      containLinkId,
    }
  });
  await deep.serial({
    operations: serialOperations
  })
  ```
  */
export async function getWebPushCertificateInsertSerialOperations(param) {
    const { deep, webPushCertificate, valueForContainForWebPushCertificate, valueForContainForUsesWebPushCertificate, shouldMakeActive = false } = param;
    const $package = new Package({ deep });
    const containerLinkId = param.containerLinkId !== null ? param.containerLinkId ?? deep.linkId : null;
    const reservedLinkIds = await getReservedLinkIds();
    const { containForWebPushCertificateLinkId: containLinkId, webPushCertificateLinkId, usesWebPushCertificateLinkId, containForUsesWebPushCertificateLinkId } = reservedLinkIds;
    const typeLinkIds = await getTypeLinkIds();
    const { containTypeLinkId, webPushCertificateTypeLinkId, usesWebPushCertificateTypeLinkId } = typeLinkIds;
    const serialOperations = [];
    const webPushCertificateInsertSerialOperation = createSerialOperation({
        type: 'insert',
        table: 'links',
        objects: {
            id: webPushCertificateLinkId,
            type_id: webPushCertificateTypeLinkId,
        },
    });
    serialOperations.push(webPushCertificateInsertSerialOperation);
    const valueOfWebPushCertificateInsertSerialOperation = createSerialOperation({
        type: 'insert',
        table: 'strings',
        objects: {
            link_id: webPushCertificateLinkId,
            value: webPushCertificate,
        },
    });
    serialOperations.push(valueOfWebPushCertificateInsertSerialOperation);
    if (containerLinkId !== null) {
        const containInsertSerialOperation = createSerialOperation({
            type: 'insert',
            table: 'links',
            objects: {
                id: containLinkId,
                type_id: containTypeLinkId,
                from_id: containerLinkId,
                to_id: webPushCertificateLinkId,
            },
        });
        serialOperations.push(containInsertSerialOperation);
        if (valueForContainForWebPushCertificate) {
            const valueForContainInsertSerialOperation = createSerialOperation({
                type: 'insert',
                table: 'strings',
                objects: {
                    link_id: containLinkId,
                    value: valueForContainForWebPushCertificate,
                },
            });
            serialOperations.push(valueForContainInsertSerialOperation);
        }
    }
    if (shouldMakeActive) {
        const usesWebPushCertificateInsertSerialOperation = createSerialOperation({
            type: 'insert',
            table: 'links',
            objects: {
                id: usesWebPushCertificateLinkId,
                type_id: usesWebPushCertificateTypeLinkId,
                from_id: deep.linkId,
                to_id: webPushCertificateLinkId,
            },
        });
        serialOperations.push(usesWebPushCertificateInsertSerialOperation);
        if (containerLinkId !== null) {
            const containForUsesWebPushCertificateInsertSerialOperation = createSerialOperation({
                type: 'insert',
                table: 'links',
                objects: {
                    id: containForUsesWebPushCertificateLinkId,
                    type_id: containTypeLinkId,
                    from_id: containerLinkId,
                    to_id: usesWebPushCertificateLinkId,
                }
            });
            serialOperations.push(containForUsesWebPushCertificateInsertSerialOperation);
            if (valueForContainForUsesWebPushCertificate) {
                const valueForContainInsertSerialOperation = createSerialOperation({
                    type: 'insert',
                    table: 'objects',
                    objects: {
                        link_id: containForUsesWebPushCertificateLinkId,
                        value: valueForContainForUsesWebPushCertificate,
                    },
                });
                serialOperations.push(valueForContainInsertSerialOperation);
            }
        }
    }
    return {
        serialOperations,
        linkIds: reservedLinkIds
    };
    async function getReservedLinkIds() {
        let result = {
            containForWebPushCertificateLinkId: 0,
            webPushCertificateLinkId: 0,
            usesWebPushCertificateLinkId: 0,
            containForUsesWebPushCertificateLinkId: 0
        };
        const linksToReserveCount = Object.keys(result).length -
            Object.keys(param.reservedLinkIds || {}).length;
        const reservedLinkIds = linksToReserveCount > 0 ? await deep.reserve(linksToReserveCount) : [];
        result = {
            containForWebPushCertificateLinkId: param.reservedLinkIds?.containForWebPushCertificateLinkId ?? reservedLinkIds.pop(),
            webPushCertificateLinkId: param.reservedLinkIds?.webPushCertificateLinkId ?? reservedLinkIds.pop(),
            usesWebPushCertificateLinkId: param.reservedLinkIds?.usesWebPushCertificateLinkId ?? reservedLinkIds.pop(),
            containForUsesWebPushCertificateLinkId: param.reservedLinkIds?.containForUsesWebPushCertificateLinkId ?? reservedLinkIds.pop(),
        };
        return result;
    }
    async function getTypeLinkIds() {
        const result = {
            containTypeLinkId: param.typeLinkIds?.containTypeLinkId ||
                (await deep.id('@deep-foundation/core', 'Contain')),
            webPushCertificateTypeLinkId: param.typeLinkIds?.webPushCertificateTypeLinkId ||
                await $package.WebPushCertificate.id(),
            usesWebPushCertificateTypeLinkId: param.typeLinkIds?.usesWebPushCertificateTypeLinkId ||
                await $package.UsesWebPushCertificate.id(),
        };
        return result;
    }
}
